/*
 (C) Copyright 2003
 Thomas Elste, info@elste.org

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
 published by the Free Software Foundation; either version 2 of
 the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 MA 02111-1307 USA
*/
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <string.h>
#include "com_ops.h"


int Open_Comport(int nPort, int baudRate, int *phFD)
{
  //int phFD;
  char portName[] = "/dev/ttyS#";  /* port's device file name template */
  struct termios tattr;            /* terminal attributes */
  char buf[32];                    /* read buffer */
  /* sanity checks... */
  //assert(phFD != NULL);
  //assert(nPort < 10);
  /* determine name of port device */
  portName[strlen(portName)-1] = '0' + (char) nPort;
  /* open terminal port... */
  *phFD = open(portName, O_RDWR | O_NOCTTY | O_NONBLOCK);
  if (*phFD < 0) return -1;
  /* set some default terminal parameters... */
  if (tcgetattr(*phFD, &tattr) < 0) return -1;  /* fetch current params */
  tattr.c_lflag &= ~(ECHO | ECHOE | ICANON | ISIG);
  tattr.c_iflag = (tattr.c_iflag & ~(INLCR | IGNCR | ICRNL)) | IGNBRK;
  tattr.c_oflag &= ~OPOST;
  /* 8N1, no flow control... */
  switch(baudRate)
  {
  	case 0:
		cfsetispeed(&tattr, B2400);
  		cfsetospeed(&tattr, B2400);
		break;
	case 1:
		cfsetispeed(&tattr, B4800);
  		cfsetospeed(&tattr, B4800);
		break;

	case 2:
		cfsetispeed(&tattr, B9600);
  		cfsetospeed(&tattr, B9600);
		break;
	case 3:
		cfsetispeed(&tattr, B19200);
  		cfsetospeed(&tattr, B19200);
		break;
	default:
		cfsetispeed(&tattr, B19200);
  		cfsetospeed(&tattr, B19200);
		break;
  }
  tattr.c_cflag = (tattr.c_cflag & ~(CSIZE | CLOCAL)) | CS8;
  tattr.c_cflag &= ~CSTOPB;
  tattr.c_cflag &= ~(PARENB | PARODD);
  tattr.c_iflag = (tattr.c_iflag & ~INPCK) | IGNPAR;
  tattr.c_cflag &= ~CRTSCTS;
  tattr.c_iflag &= ~(IXON | IXOFF);
  /* NOTE: the following two instructions configure read() to return
           immediately with as much data (even none) as available */
  tattr.c_cc[VMIN] = 0;
  tattr.c_cc[VTIME] = 0;
  if (tcsetattr(*phFD, TCSAFLUSH, &tattr) < 0) return -1;   /* set new params */
  /* flush buffers... */
  tcflush(*phFD, TCIFLUSH);
  tcflush(*phFD, TCOFLUSH);

  return 0;
}


void Close_Comport (int fd)
{
	close(fd);
}


void Read_Byte(int com_dev,unsigned char *res)
{
	int r;
	//int test=0;
	res[0]=0;
	r=read(com_dev,res,1);
	if(r==-1) perror("read():");
//	printf("get byte: %x, read: %i\n",*res,r);
}

int Write_Byte(int com_dev, unsigned char to_send)
{
	int ret;
	ret=write(com_dev,&to_send,1);
//	printf("write said: %i\n",ret);
	return(ret);
}
