/*
 (C) Copyright 2003
 Thomas Elste, info@elste.org

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
 published by the Free Software Foundation; either version 2 of
 the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 MA 02111-1307 USA
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <string.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qlcdnumber.h>
#include <qfont.h>
#include <qpainter.h>
#include <qlineedit.h>
#include <qtooltip.h>
#include <qlabel.h>
#include <qslider.h>
#include <qprogressbar.h>
#include <qlayout.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qdialog.h>

#include "com_ops.h"
#include "pwm_gui.h"

#define SIZE_H 300
#define SIZE_W 500
#define ELEMENT_H 20
#define ELEMENT_H_SPACE 10
#define ELEMENT_V_SPACE 10
#define BUTTON_W 80
#define LCD_W 30
#define LABEL_W 50
#define SLIDER_W 90
#define BAR_W 80
#define SLIDER_H 15

#define PWM_MIN_VALUE 1
#define PWM_MAX_VALUE 100
#define PWM_CHANNELS 4

#define SLEEP_TIME 10000

#define VERSION "0.5.4"

char dir[255];

class PWMProfile: public QObject
{
Q_OBJECT
public:
	PWMProfile();
	void get_profile(int nr, int *prof);
	void get_profile(int nr, int *prof, char *pname);
	int get_profile_nr();
	void read_config();

public slots:
	void change_channel1(int val);
	void change_channel2(int val);
	void change_channel3(int val);
	void change_channel4(int val);
	void write_akt_profile(char* nname);
	void load_profile(int val);

signals:
	void emite_ttynr(int);
	void emite_baudrate(int);
	void emite_new_profile(char*);
	void emite_channel1(int);
	void emite_channel2(int);
	void emite_channel3(int);
	void emite_channel4(int);

private:
	int akt_prof, profile_count,load_on_start;
	chprofile_t akt_profile;
	chprofile_t *profile;


};

PWMProfile::PWMProfile()
{
}

void PWMProfile::change_channel1(int val)
{
	akt_profile.channel[0]=val;
}

void PWMProfile::change_channel2(int val)
{
	akt_profile.channel[0]=val;
}

void PWMProfile::change_channel3(int val)
{
	akt_profile.channel[0]=val;
}

void PWMProfile::change_channel4(int val)
{
	akt_profile.channel[0]=val;
}

void PWMProfile::write_akt_profile(char* nname)
{
	strcpy(akt_profile.name,nname);
}

void PWMProfile::load_profile(int val)
{
	if((val<0)||(val>=profile_count)) return;
	printf("profile gewhlt: %i. %s\n",val, profile[val].name);
	emite_channel1(profile[val].channel[0]);
	emite_channel2(profile[val].channel[1]);
	emite_channel3(profile[val].channel[2]);
	emite_channel4(profile[val].channel[3]);
}

void PWMProfile::read_config()
{
	FILE *confFile;
	int readval,i,j;
	confFile=fopen(CONF_FILE,"r");
	if(confFile==NULL)
	{
		chdir(dir);
		printf("%s\n",dir);
		confFile=fopen(CONF_FILE,"r");
	}
	if(confFile!=NULL)
	{
		printf("reading from config file\n");
		if(fscanf(confFile,"%i",&readval)>0) emite_ttynr(readval);
		else emite_ttynr(0);
		if(fscanf(confFile,"%i",&readval)>0) emite_baudrate(readval);
		else emite_ttynr(0);
		if(fscanf(confFile,"%i",&readval)>0) profile_count=readval;
		else profile_count=0;
		if(fscanf(confFile,"%i",&readval)>0) akt_prof=readval;
		else akt_prof=-1;
		if(fscanf(confFile,"%i",&readval)>0) load_on_start=readval;
		else load_on_start=-1;
		profile=(chprofile_t*)calloc(profile_count,sizeof(chprofile_t));
		printf("reading profiles\n");
		for(i=0;i<profile_count;i++)
		{
			fscanf(confFile,"%s",profile[i].name);
			printf("reading ..");
			for(j=0;j<4;j++) if(fscanf(confFile,"%i", &(profile[i].channel[j]))<=0) profile[i].channel[j]=100;
//			printf("%i\n",profile[i].channel[i]);
//			printf("%s\n",profile[i].name);
			emite_new_profile(profile[i].name);

		}
		printf("finished reading profiles\n");
		fclose(confFile);
	}
	else
	{
		printf("no config file found, using default values for initialisation\n");
		emite_ttynr(0);
		emite_baudrate(0);
		profile_count=0;
		akt_prof=-1;
		load_on_start=-1;
	}
}


class PWMData : public QObject
{
Q_OBJECT
public:
	PWMData();
protected:
	int getmin(unsigned char *data,int numb);
	int getmax(unsigned char *data,int numb);
	void emite_channel_vals();
	void abs_to_diff();
	void diff_to_abs();

public slots:
	void change_channel1(int val);
	void change_channel2(int val);
	void change_channel3(int val);
	void change_channel4(int val);
	void send_data();
	void get_data();
	void change_tty(int val);
	void change_speed(int val);

signals:
	void progress(int);
	void channel_1(int);
	void channel_2(int);
	void channel_3(int);
	void channel_4(int);

private:
	unsigned char channel[PWM_CHANNELS];
	unsigned char counters[PWM_CHANNELS];
	unsigned char order[PWM_CHANNELS];
	unsigned char overhead;
	int ttyNr;
	int speedNr;

};

PWMData::PWMData()
{
	int i;
	for(i=0;i<4;i++)
	{
		channel[i]=0;
		counters[i]=0;
		order[i]=0;
	}
	overhead=0;
}

int PWMData::getmin(unsigned char *data,int numb)
{
	int ret=0,i;
	for(i=1;i<numb;i++)
	{
		if(data[i]<data[ret]) ret=i;
	}
	return(ret);
}

int PWMData::getmax(unsigned char *data,int numb)
{
	int ret=0,i;
	for(i=1;i<numb;i++)
	{
		if(data[i]>data[ret]) ret=i;
	}
	return(ret);
}

void PWMData::emite_channel_vals()
{
	channel_1(channel[0]);
	channel_2(channel[1]);
	channel_3(channel[2]);
	channel_4(channel[3]);
}

void PWMData::change_channel1(int val)
{
	channel[0]=(unsigned char)val;
	channel_1(channel[0]);
	printf("neuer wert fr %i. kanal: %i\n",0,channel[0]);
}

void PWMData::change_channel2(int val)
{
	channel[1]=(unsigned char)val;
	channel_2(channel[1]);
	printf("neuer wert fr %i. kanal: %i\n",1,channel[1]);
}

void PWMData::change_channel3(int val)
{
	channel[2]=(unsigned char)val;
	channel_3(channel[2]);
	printf("neuer wert fr %i. kanal: %i\n",2,channel[2]);
}

void PWMData::change_channel4(int val)
{
	channel[3]=(unsigned char)val;
	channel_4(channel[3]);
	printf("neuer wert fr %i. kanal: %i\n",3,channel[3]);
}

void PWMData::change_tty(int val)
{
	ttyNr=val;
	printf("neuer tty: %i\n",ttyNr);
}

void PWMData::change_speed(int val)
{
	speedNr=val;
	printf("neuer speed: %i\n",speedNr);
}


void PWMData::abs_to_diff()
{
	int idx1,i,j,k;
	unsigned char local_order[PWM_CHANNELS];
	unsigned char local_data[PWM_CHANNELS];

	memcpy(local_data,channel,PWM_CHANNELS*sizeof(char));
	for(j=0;j<PWM_CHANNELS;j++)
	{
		idx1=getmin(local_data,PWM_CHANNELS);
		if(j>0) order[j]=(1<<idx1)|order[j-1];
		else order[j]=(1<<idx1);
		local_data[idx1]=0xff;
	}
	memcpy(local_data,channel,PWM_CHANNELS*sizeof(char));
	for(j=0;j<PWM_CHANNELS;j++)
	{
		idx1=getmin(local_data,PWM_CHANNELS);
		if(j>0)
		{
			counters[j]=local_data[idx1];
			for(k=0;k<j;k++) counters[j]-=counters[k];
		}
		else counters[j]=local_data[idx1];
		local_data[idx1]=0xff;
	}

	for(j=0;j<PWM_CHANNELS;j++)
	{
		if(counters[j]==0) counters[j]=1;
		printf("%i\n",channel[j]);
	}


	overhead=PWM_MAX_VALUE-channel[getmax(channel,PWM_CHANNELS)];
	if(overhead==0) overhead=1;
}

void PWMData::diff_to_abs()
{
	int i,nidx,ch_nr=0,old_ch_nr,j;
	unsigned char old_order;
	unsigned char local_order[PWM_CHANNELS];
	//local_order=(unsigned char*)malloc(sizeof(char));
	memcpy(local_order,order,PWM_CHANNELS*sizeof(char));
	for(i=0;i<PWM_CHANNELS;i++)
	{
		//todo:minimum aus pwm entfernen (in kopie)
		nidx=getmin(local_order,PWM_CHANNELS);
		if(i>0)
		{
			ch_nr=(int)((log(order[nidx]^old_order)/log(2))+0.5);
			channel[ch_nr]=counters[nidx]+channel[old_ch_nr];
		}
		else
		{
			ch_nr=(int)((log(order[nidx])/log(2))+0.5);
			channel[ch_nr]=counters[nidx];
		}
		old_order=order[nidx];
		local_order[nidx]=0xff;
		old_ch_nr=ch_nr;
	}
}

void PWMData::send_data()
{
	int com_fd,i;
	unsigned char dummy[1];
	abs_to_diff();
	printf("vor com\n");
	Open_Comport (ttyNr,speedNr,&com_fd);
	for(i=3;i>-1;i--)
	{
		Write_Byte(com_fd,order[i]);
		progress(4-i);
		usleep(SLEEP_TIME);
		Read_Byte(com_fd,dummy);
		usleep(SLEEP_TIME);
		progress(4-i);
	}
	
	Write_Byte(com_fd,overhead);
	usleep(SLEEP_TIME);
	Read_Byte(com_fd,dummy);
	usleep(SLEEP_TIME);
	progress(5);
	
	for(i=3;i>-1;i--)
	{
		Write_Byte(com_fd,counters[i]);
		progress(4-i);
		usleep(SLEEP_TIME);
		Read_Byte(com_fd,dummy);
		usleep(SLEEP_TIME);
		progress(9-i);
	}
	
	Close_Comport(com_fd);
	progress(0);

	get_data();

}

void PWMData::get_data()
{
	int com_fd,i;
	unsigned char rbyte[9]={0};
	Open_Comport (ttyNr,speedNr, &com_fd);
	Write_Byte(com_fd,0);
	for(i=0;i<9;i++)
	{
		usleep(SLEEP_TIME);
		Read_Byte(com_fd,rbyte+i);
	}
	for(i=0;i<9;i++) printf("%i: %x\n",i,rbyte[i]);
	Close_Comport(com_fd);

	for(i=3;i>-1;i--)
	{
		order[i]=rbyte[i] & 0xf;
		progress(4-i);
		usleep(SLEEP_TIME);
	}

	overhead=rbyte[4];
	usleep(SLEEP_TIME);
	progress(5);
	for(i=3;i>-1;i--)
	{
		counters[i]=rbyte[i+5];
		progress(9-i);
		usleep(SLEEP_TIME);
	}

	progress(0);
	diff_to_abs();
	for(i=0;i<4;i++) printf("new Channel val: %i\n",channel[i]);
	emite_channel_vals();
}


class OptionsWidget: public QDialog
{
Q_OBJECT
public:
	OptionsWidget(QWidget *parent=0, const char *name=0, bool modal=FALSE, WFlags f=0);
	QComboBox *sSpeed, *stty;

public slots:
	void set_item_tty(int);
	void set_item_baud(int);

private:
	QPushButton *quit,*ok;
	QLabel *speedLabel, *ttyLabel;

};

OptionsWidget::OptionsWidget(QWidget *parent, const char *name, bool modal, WFlags f)
        : QDialog( parent, name, modal, f )
{
	QVBoxLayout *globalLayout=new QVBoxLayout(this,6,11);
		QHBoxLayout *hLayout1=new QHBoxLayout(11);
		QHBoxLayout *hLayout2=new QHBoxLayout(11);
		QHBoxLayout *hLayout3=new QHBoxLayout(11);

	globalLayout->addLayout(hLayout1);
	globalLayout->addLayout(hLayout2);
	globalLayout->addLayout(hLayout3);

	speedLabel=new QLabel(this,"speeLabel");
	speedLabel->setText("Baudrate:");
	ttyLabel=new QLabel(this,"ttyLabel");
	ttyLabel->setText("Schnittstelle:");

	sSpeed=new QComboBox(this,"sSpeed");
	sSpeed->setEditable(false);
	sSpeed->insertItem("2400",0);
	sSpeed->insertItem("4800",1);
	sSpeed->insertItem("9600",2);
	sSpeed->insertItem("19200",3);

	stty=new QComboBox(this,"stty");
	stty->setEditable(false);
	stty->insertItem("/dev/ttyS0",0);
	stty->insertItem("/dev/ttyS1",1);
	stty->insertItem("/dev/ttyS2",2);
	stty->insertItem("/dev/ttyS3",3);

	quit=new QPushButton("Abbrechen",this,"quit");
	ok=new QPushButton("Ok",this,"ok");

	hLayout1->addWidget(ttyLabel);
	hLayout1->addWidget(stty);

	hLayout2->addWidget(speedLabel);
	hLayout2->addWidget(sSpeed);

	hLayout3->addWidget(ok);
	hLayout3->addWidget(quit);
	


	connect( quit, SIGNAL(clicked()), this, SLOT(reject()));
	connect( ok, SIGNAL(clicked()), this, SLOT(accept()));
}

void OptionsWidget::set_item_tty(int val)
{
	stty->setCurrentItem(val);
}

void OptionsWidget::set_item_baud(int val)
{
	sSpeed->setCurrentItem(val);
}


class MyWidget : public QWidget
{
Q_OBJECT
public:
	MyWidget( QWidget *parent=0, const char *name=0 );
	int getOptionsTty();
	int getOptionsSpeed();


public slots:
	void insert_profile(char*);

protected:
	void resizeEvent( QResizeEvent * );
	void paintEvent(QPaintEvent *);
	void timerEvent(QTimerEvent *);
	void mouseMoveEvent(QMouseEvent *);
private:
	QPushButton *quit,*check,*transm,*opt;
	QLabel *ch1,*ch2,*ch3,*ch4,*chlabel[4];
	PWMData *pwm;
	PWMProfile *profiles;
	QSlider *s_ch1, *s_ch2, *s_ch3, *s_ch4,*chslider[4];
	QLCDNumber *l_ch1,*chlcd[4];
	QComboBox *profilBox;
	QProgressBar *p_ch1,*chbar[4],*big_bar;
	QLineEdit *line_1,*line_2;
	OptionsWidget *options;
	int i,j;
};

MyWidget::MyWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
	i=0;j=0;
	char dummy[32];
	pwm=new PWMData();
	options=new OptionsWidget(this,"test",TRUE,0);
	profiles=new PWMProfile();

	//options.repaint();
	//options.show();

//	this.setBaseSize(SIZE_W,SIZE_H);

	QVBoxLayout *globalLayout=new QVBoxLayout(this,6,11);

		QVBoxLayout *topLayout=new QVBoxLayout(6);

			QHBoxLayout *topLine1Layout=new QHBoxLayout(11);
			QHBoxLayout *topLine2Layout=new QHBoxLayout(11);

		QHBoxLayout *middleLayout=new QHBoxLayout(11);

			QVBoxLayout *labelLayout=new QVBoxLayout(6);
			QVBoxLayout *scrollLayout=new QVBoxLayout(6);
			QVBoxLayout *lcdLayout=new QVBoxLayout(6);
			QVBoxLayout *progressLayout=new QVBoxLayout(6);

		QHBoxLayout *buttonBox = new QHBoxLayout(11);

        globalLayout->addLayout(topLayout);

	globalLayout->addLayout(middleLayout);
	globalLayout->addLayout(buttonBox);

        topLayout->addLayout(topLine1Layout);
	topLayout->addLayout(topLine2Layout);
	
	middleLayout->addLayout(labelLayout);
	middleLayout->addLayout(scrollLayout);
	middleLayout->addLayout(lcdLayout);
	middleLayout->addLayout(progressLayout);	
	

	QLabel *prof_label=new QLabel(this,"prof_label");
	prof_label->setText("Profil:");
	profilBox=new QComboBox(this,"profilBox");
	//profilBox->setEditable(TRUE);
	profilBox->setEditable(false);
	profilBox->setDuplicatesEnabled(TRUE);
	/*profilBox->insertItem("dummy1",0);
	profilBox->insertItem("dummy2",1);
	profilBox->insertItem("dummy3",2);
	profilBox->insertItem("dummy4",3);
	profilBox->insertItem("dummy5",4);*/
	QCheckBox *startProfil=new QCheckBox(this,"startProfil");
	startProfil->setText("beim Start laden");
	
	profilBox->setMaximumHeight(25);
	
	
	topLine1Layout->addWidget(prof_label);
	topLine1Layout->addWidget(profilBox);
	topLine1Layout->addWidget(startProfil);	
	
	
	QPushButton *profLoad = new QPushButton( "Laden", this, "profLoad" );
	QPushButton *profSave = new QPushButton( "Speichern", this, "profSave" );
	QPushButton *profDel = new QPushButton( "Lschen", this, "profDel" );	

	profLoad->setMaximumHeight(25);
	profSave->setMaximumHeight(25);
	profDel->setMaximumHeight(25);
	
	topLine2Layout->addWidget(profLoad);
	topLine2Layout->addWidget(profSave);
	topLine2Layout->addWidget(profDel);
	
	


//	setForegroundColor(QColor(0,255,0));
	for(i=1;i<5;i++)
	{
		sprintf(dummy,"chlabel%i",i);
		chlabel[i-1]=new QLabel(this,dummy);
		sprintf(dummy,"Kanal %i",i);
		chlabel[i-1]->setText(dummy);
		labelLayout->addWidget(chlabel[i-1]);
	
		sprintf(dummy,"chslider%i",i);
		chslider[i-1]=new QSlider(chslider[i-1]->Horizontal,this,dummy);
		chslider[i-1]->setRange(PWM_MIN_VALUE,PWM_MAX_VALUE);
		chslider[i-1]->setMaximumHeight(15);
		chslider[i-1]->setMinimumHeight(15);
		scrollLayout->addWidget(chslider[i-1]);

                sprintf(dummy,"chlcd%i",i);
                chlcd[i-1]=new QLCDNumber(2,this,dummy);
		chlcd[i-1]->setSegmentStyle(chlcd[i-1]->Filled);
		chlcd[i-1]->setMaximumSize(40,50);
//		chlcd[i-1]->setMinimumSize(25,25);		
		lcdLayout->addWidget(chlcd[i-1]);

		connect( chslider[i-1], SIGNAL(valueChanged(int)), chlcd[i-1], SLOT(display (int)) );
        		
		sprintf(dummy,"chbar%i",i);
		chbar[i-1]=new QProgressBar(PWM_MAX_VALUE,this,dummy);
		chbar[i-1]->setMaximumHeight(15);
		chbar[i-1]->setMinimumHeight(15);
		progressLayout->addWidget(chbar[i-1]);
	}


	check = new QPushButton( "Auslesen", this, "check" );
	transm=new QPushButton("bertragen",this,"transm");
	opt=new QPushButton("Einstellungen",this,"opt");	
	quit=new QPushButton("Beenden",this,"quit");
	
	check->setMaximumHeight(25);
	transm->setMaximumHeight(25);
	opt->setMaximumHeight(25);
	quit->setMaximumHeight(25);
	
	check->setMinimumSize(90,25);
	transm->setMinimumSize(90,25);
	opt->setMinimumSize(90,25);
	quit->setMinimumSize(90,25);
	
	QToolTip::add(check,"liest die aktuell eingestellten Werte aus der PWM Steuerung");
	QToolTip::add(transm,"bertrgt die eingestellten Werte zur PWM Steuerung");
	QToolTip::add(quit,"Programm Beenden");

	
	buttonBox->addWidget(check);
	buttonBox->addWidget(transm);
	buttonBox->addWidget(opt);	
	buttonBox->addWidget(quit);


	big_bar=new QProgressBar(9,this,"big_bar");
	big_bar->setMaximumHeight(15);
	globalLayout->addWidget(big_bar);
		
	connect( chslider[0], SIGNAL(valueChanged(int)), pwm, SLOT(change_channel1(int)));
	connect( chslider[1], SIGNAL(valueChanged(int)), pwm, SLOT(change_channel2(int)));
	connect( chslider[2], SIGNAL(valueChanged(int)), pwm, SLOT(change_channel3(int)));
	connect( chslider[3], SIGNAL(valueChanged(int)), pwm, SLOT(change_channel4(int)));

	connect( chslider[0], SIGNAL(valueChanged(int)), profiles, SLOT(change_channel1(int)));
	connect( chslider[1], SIGNAL(valueChanged(int)), profiles, SLOT(change_channel2(int)));
	connect( chslider[2], SIGNAL(valueChanged(int)), profiles, SLOT(change_channel3(int)));
	connect( chslider[3], SIGNAL(valueChanged(int)), profiles, SLOT(change_channel4(int)));

	connect( pwm, SIGNAL(channel_1(int)),chslider[0] , SLOT(setValue(int)));
	connect( pwm, SIGNAL(channel_2(int)),chslider[1] , SLOT(setValue(int)));
	connect( pwm, SIGNAL(channel_3(int)),chslider[2] , SLOT(setValue(int)));
	connect( pwm, SIGNAL(channel_4(int)),chslider[3] , SLOT(setValue(int)));

	connect( pwm, SIGNAL(channel_1(int)),chbar[0] , SLOT(setProgress(int)));
	connect( pwm, SIGNAL(channel_2(int)),chbar[1] , SLOT(setProgress(int)));
	connect( pwm, SIGNAL(channel_3(int)),chbar[2] , SLOT(setProgress(int)));
	connect( pwm, SIGNAL(channel_4(int)),chbar[3] , SLOT(setProgress(int)));

	connect( pwm, SIGNAL(progress(int)), big_bar, SLOT(setProgress(int)));

	connect( check, SIGNAL(clicked()), pwm, SLOT(get_data()));
	connect( transm, SIGNAL(clicked()), pwm, SLOT(send_data()));
	connect( opt, SIGNAL(clicked()), options, SLOT(exec()));
	connect( quit, SIGNAL(clicked()), qApp, SLOT(quit()) );

	connect( options->stty, SIGNAL(activated(int)), pwm, SLOT(change_tty(int)));
	connect( options->sSpeed, SIGNAL(activated(int)),pwm, SLOT(change_speed(int)));
	
	connect( profiles, SIGNAL(emite_ttynr(int)),options,SLOT(set_item_tty(int)));
	connect( profiles, SIGNAL(emite_baudrate(int)),options,SLOT(set_item_baud(int)));

	connect( profiles, SIGNAL(emite_ttynr(int)),pwm, SLOT(change_tty(int)));
	connect( profiles, SIGNAL(emite_baudrate(int)),pwm, SLOT(change_speed(int)));
	
	connect( profiles, SIGNAL(emite_new_profile(char*)),this,SLOT(insert_profile(char*)));
	
	connect( profiles, SIGNAL(emite_channel1(int)), pwm, SLOT(change_channel1(int)));
	connect( profiles, SIGNAL(emite_channel2(int)), pwm, SLOT(change_channel2(int)));
	connect( profiles, SIGNAL(emite_channel3(int)), pwm, SLOT(change_channel3(int)));
	connect( profiles, SIGNAL(emite_channel4(int)), pwm, SLOT(change_channel4(int)));

	connect( profilBox, SIGNAL(activated(int)), profiles, SLOT(load_profile(int)));

	profiles->read_config();
	printf("initialisation complete\n");
	j=0;

}

int MyWidget::getOptionsTty()
{
	return(options->stty->currentItem());
}

int MyWidget::getOptionsSpeed()
{
	return(options->sSpeed->currentItem());
}

void MyWidget::insert_profile(char* name)
{
	profilBox->insertItem(name);
}


void MyWidget::resizeEvent( QResizeEvent * )
{
}

void MyWidget::paintEvent(QPaintEvent*)
{
	QPainter p;
	p.begin(this);
	p.setBrush( QColor(i%255,j%255,(i+j)%255) );
	p.setPen( NoPen );
    	//i+=1;
    	p.drawEllipse(i,j,i,j);			
	p.end();
}

void MyWidget::timerEvent(QTimerEvent*)
{
	update();
}

void MyWidget::mouseMoveEvent(QMouseEvent *e)
{
}


int main( int argc, char **argv )
{
	QApplication a( argc, argv );
	printf("%s\n",argv[0]);
	memcpy(dir,argv[0],(strlen(argv[0])-(strlen(strrchr(argv[0], '/'))))>254?0:(strlen(argv[0])-(strlen(strrchr(argv[0], '/')))));
	MyWidget w;

	w.repaint();
	a.setMainWidget( &w );
	w.show();
	return a.exec();
}

#include "myclass.moc"
